clearvars; clc;

% ============= VHODNI PARAMETRI =============
file_in = "podatki\tocke_llh_deg.txt";
crd_in = "llh_deg";    % podprti tipi koordinat ["xyz", "llh_deg", "llh_dms"]
crd_out = "xyz";       % podprti tipi koordinat ["xyz", "llh_deg", "llh_dms"]
% ============================================

if ~isfile(file_in)
    error("Vhodna datoteka ne obstaja.");
end

crd_types = ["xyz", "llh_deg", "llh_dms"];

if ~any(matches(crd_types, crd_in))
    error("Napacen vhodni format koordinat. Veljavne opcije: xyz, llh_deg, llh_dms.")
end

if ~any(matches(crd_types, crd_out))
    error("Napacen izhodni format koordinat. Veljavne opcije: xyz, llh_deg, llh_dms.")
end

if crd_in == crd_out
    warning("Vhodni in izhodni tip koordinat je enak.")
    return
end

% sestavi ime izhodne datoteke
[filepath_in, filename_in, fileext_in] = fileparts(file_in);
filepath_out = filepath_in;
filename_out = filename_in + "_v_" + crd_out + ".txt";
file_out = fullfile(filepath_out, filename_out);

% preberi vhodne podatke v tabelo
points_in = readtable(file_in, NumHeaderLines=1);
n_points = size(points_in, 1);

try
    fid = fopen(file_out, 'w');
    
    switch crd_in
        
        case "xyz" 
            points_in.Properties.VariableNames = {'name', 'X', 'Y', 'Z'};
            process_xyz(points_in, crd_out, fid, n_points);
        
        case "llh_deg"
            points_in.Properties.VariableNames = {'name', 'lat', 'lon', 'h'};
            process_llh_deg(points_in, crd_out, fid, n_points);
        
        case "llh_dms"
            points_in.Properties.VariableNames = {'name', 'lat', 'lon', 'h'};
            process_llh_dms(points_in, crd_out, fid, n_points);
        
        % Napačen tip koordinat ujamemo že na začetku programa. Alternativno bi lahko uporabili tudi
        % otherwise v switch stavku (spodnji dve vrstici).
        % otherwise
        %     error("Napacen vhodni format koordinat. Veljavne opcije: xyz, llh_deg, llh_dms.")
    end

    fclose(fid);

catch ME
    % zapri izhodno datoteko
    if exist('fid', 'var') && fid > 0
        fclose(fid);
    end
    % pobrisi izhodno datoteko
    if exist('file_out', 'var') && isfile(file_out)
        delete(file_out);
    end
    rethrow(ME);
end


% ======================================
% ========== LOKALNE FUNKCIJE ==========
% ======================================

% Funkcijo lahko definiramo tudi znotraj posamezne skripte. Takšno funkcijo imenujemo 
% lokalna funckija (local function) in je na voljo za uporabo (jo lahko kličemo) zgolj
% znotraj skripte, v kateri je funkcija definirana.
% Lokalne funkcije uporabljamo, če se nek del iste kode znotraj ene skirpte ponovi večkrat
% (vsebina funkcije je vezana zgolj na to skripto), ali pa zgolj za bolšo berljivost kode.

function process_xyz(points_in, crd_out, file_out_fid, n_points)
    % vhodne koordinate so xyz
    
    switch crd_out
        case "llh_deg"
            % xyz -> llh_deg
            fprintf(file_out_fid, "# pt        lat [°]           lon [°]          h [m]\n");
            for i = 1 : n_points
                pt_llh_deg = xyz2sferne([points_in.X(i), points_in.Y(i), points_in.Z(i)]);
                fprintf(file_out_fid, "%4s    %14.10f    %15.10f    %9.4f\n", points_in.name{i}, pt_llh_deg);
            end
        
        case "llh_dms"
            % xyz -> llh_dms
            fprintf(file_out_fid, "# pt    lat [ddd.mmssss]     lon [ddd.mmssss]      h [m]\n");
            for i = 1 : n_points
                pt_llh_deg = xyz2sferne([points_in.X(i), points_in.Y(i), points_in.Z(i)]);
                pt_llh_dms = [deg2dddmmssss(pt_llh_deg(1), 8), deg2dddmmssss(pt_llh_deg(2), 8), pt_llh_deg(3)];
                fprintf(file_out_fid, "%4s    %16.12f    %17.12f    %9.4f\n", points_in.name{i}, pt_llh_dms);
            
            end
    end
end


function process_llh_deg(points_in, crd_out, file_out_fid, n_points)
    % vhodne koordinate so llh_deg
    
    switch crd_out
        case "xyz"
            % llh_deg -> xyz
            fprintf(file_out_fid, "# pt        X [m]            Y [m]            Z [m]\n");
            for i = 1 : n_points
                pt_llh_deg = [points_in.lat(i), points_in.lon(i), points_in.h(i)];
                pt_xyz = sferne2xyz(pt_llh_deg);
                fprintf(file_out_fid, "%4s    %13.4f    %13.4f    %13.4f\n", points_in.name{i}, pt_xyz);
            end
        
        case "llh_dms"
            % llh_deg -> llh_dms
            fprintf(file_out_fid, "# pt    lat [ddd.mmssss]     lon [ddd.mmssss]      h [m]\n");
            for i = 1 : n_points
                pt_llh_dms = [deg2dddmmssss(points_in.lat(i), 8), deg2dddmmssss(points_in.lon(i), 8), points_in.h(i)];
                fprintf(file_out_fid, "%4s    %16.12f    %17.12f    %9.4f\n", points_in.name{i}, pt_llh_dms);
            end
    end
end


function process_llh_dms(points_in, crd_out, file_out_fid, n_points)
    % vhodne koordinate so llh_dms
    
    switch crd_out
        case "xyz"
            % llh_dms -> xyz
            fprintf(file_out_fid, "# pt        X [m]            Y [m]            Z [m]\n");
            for i = 1 : n_points
                pt_llh_deg = [dddmmssss2deg(points_in.lat(i)), dddmmssss2deg(points_in.lon(i)), points_in.h(i)];
                pt_xyz = sferne2xyz(pt_llh_deg);
                fprintf(file_out_fid, "%4s    %13.4f    %13.4f    %13.4f\n", points_in.name{i}, pt_xyz);
            end

        case "llh_deg"
            % llh_dms -> llh_deg
            fprintf(file_out_fid, "# pt        lat [°]           lon [°]          h [m]\n");
            for i = 1 : n_points
                pt_llh_deg = [dddmmssss2deg(points_in.lat(i)), dddmmssss2deg(points_in.lon(i)), points_in.h(i)];
                fprintf(file_out_fid, "%4s    %14.10f    %15.10f    %9.4f\n", points_in.name{i}, pt_llh_deg);
            end
    end
end
